import threading


class _dbBase:
    """
        code follow DBAPI-2.0 https://www.python.org/dev/peps/pep-0249/
    """

    def __init__(self, dbModule, **dbInfo):
        self._dbModule = dbModule
        self._dbInfo = dbInfo
        self._status = threading.Lock()
        self._conn = self._connect()

    def _connect(self):
        if self._status.acquire(blocking=False):
            while True:
                try:
                    return self._dbModule.connect(**self._dbInfo)
                except Exception as e:
                    print(type(e), e)
                finally:
                    self._status.release()
        else:
            self._status.acquire()
            self._status.release()
            return self._conn

    def _check(self):
        try:
            cursor = self._conn.cursor()
            cursor.execute('select 1')
            if cursor.fetchone()[0] != 1:
                raise Exception('select error!')
        except Exception as e:
            import logging
            logging.getLogger().warning(f'数据库错误,正在重连! {e}')
            self._conn = self._connect()
            self._check()

    def execute(self, query, vals=None, *args, **kwargs):
        self._check()
        cursor = self._conn.cursor()
        cursor.execute(query, vals, *args, **kwargs)
        self._conn.commit()
        return cursor

    def close(self):
        self._conn.close()

    def __del__(self):
        if self._conn:
            self.close()
