import base64

import httpx


class trueCaptcha:
    def __init__(self, userId, apiKey):
        self.userId = userId
        self.apiKey = apiKey

    def solver(self, captcha_image_url: str, session=httpx, case='mixed', mode='default') -> dict:
        # case sensitivity of text (upper | lower| mixed)
        # Human solving only for paid users
        response = session.get(captcha_image_url)
        encoded_string = base64.b64encode(response.content)
        return self.solverB64(encoded_string.decode(), case, mode)

    def solverB64(self, imgB64Str: str, case='mixed', mode='default') -> dict:
        # case sensitivity of text (upper | lower| mixed)
        # Human solving only for paid users
        """
        TrueCaptcha API doc: https://apitruecaptcha.org/api
        Free to use 100 requests per day.
        -- response::
        {
            "result": "", ==> Or "result": 0
            "conf": 0.85,
            "usage": 0,
            "requestId": "ed0006e5-69f0-4617-b698-97dc054f9022",
            "version": "dev2"
        }
        """
        url = "https://api.apitruecaptcha.org/one/gettext"
        data = {
            "userid": self.userId,
            "apikey": self.apiKey,
            # case sensitivity of text (upper | lower| mixed)
            "case": case,
            # use human or AI (human | default)
            "mode": mode,
            "data": imgB64Str,
        }
        r = httpx.post(url=url, json=data)
        return r.json()

    def usage(self) -> dict:
        # get_captcha_solver_usage
        # [{'count': '5', 'date': '2022-02-15', 'userid': 'xxx'}]
        # {'message': 'Internal server error'} 账号不存在
        # 每天100个免费
        url = "https://api.apitruecaptcha.org/one/getusage"

        params = {
            "username": self.userId,
            "apikey": self.apiKey,
        }
        r = httpx.get(url=url, params=params)
        return r.json()

    def check(self) -> bool:
        """
        :return:True代表账号正常,False为账号异常
        """
        if 'Internal server error' in str(self.usage()):
            return False
        else:
            return True

    @staticmethod
    def calc(exp: str):
        return eval(exp.lower().replace('x', '*'))
